<?php
/**
 * Bicycle Libraries
 *
 * Set of libraries wich can help you
 *
 * @package   Bicycle Libraries
 * @author    Konstantin.Myakshin
 * @copyright Copyright (c) 2009-2010, Brouzie, Inc.
 * @license   LGPL
 * @link      http://brouzie.com/projects/bicycle-libraries/
 * @link      http://code.google.com/p/bicycle-libraries/
 * @filesource
 */

class BL_Application
{
	const BACKEND   = 'backend';
	const FRONTEND  = 'frontend';

	private static $instance = NULL;

	private static $appType;

	protected $settings = array();

	protected $modules = array();
	// current active module with submodules
	protected $module  = array();
	/**
	 *
	 * @var BL_Request
	 */
	protected $request;
	/**
	 *
	 * @var BL_Response
	 */
	protected $response;
	/**
	 *
	 * @var BL_Page
	 */
	protected $page;
	/**
	 *
	 * @var BL_Event_Dispatcher
	 */
	protected $eventDispatcher;

	protected $context;

	protected $autoRender = true;

	protected function __construct(BL_Context $context)
	{
		$this->context = $context->injectApp($this);
		BL_Settings::setPathes(include 'config/application/pathes.php');

		$this->request  = $context->request();
		$this->response = $context->response();
		$this->page     = $context->page();
		
		$this->settings = BL_Settings::get('application.main', self::$appType);
		$this->modules  = $this->prepareModules();

		$this->eventDispatcher = $context->eventDispatcher();

		//TODO: $context->getRouter()->parse();
		$routeParsers = BL_Settings::get('application.routing', self::$appType);
		BL_Router::parse($this->request, NULL, $routeParsers);
	}

	protected function prepareModules()
	{
		return BL_Settings::get('application.modules', self::$appType);
	}

	/**
	 *
	 * @return BL_Application
	 */
	final public static function instance(BL_Context $context = NULL)
	{
		return self::$instance ? self::$instance : self::$instance = new self($context);
	}

	//public static function run($type = self::FRONTEND)
	public static function run(BL_Context $context, $type = self::FRONTEND)
	{
		if (self::$instance)
			throw new BL_Exception('Application already running');

		self::$appType = $type == self::BACKEND ? self::BACKEND : self::FRONTEND;

		$_self = self::instance($context);
		$_self->dispatchModule();

		if ($_self->autoRender) {
			$_self->shutDown();
		}
	}

	public static function getAppType()
	{
		return self::$appType;
	}

	public static function isFrontend()
	{
		return self::$appType == self::FRONTEND;
	}

	public static function isBackend()
	{
		return self::$appType == self::BACKEND;
	}

	public function lib($libName, $type = NULL)
	{
		$type || $type = 'default';

		switch ($libName) {
			case 'db':
				return $this->context->db($type);

			case 'user':
				return $this->context->user();

			case 'users':
				return $this->context->users();

			default:
				throw new BL_Exception('Not implemented yet');
		}
	}
	/**
	 *
	 * @return BL_Context
	 */
	public function getContext()
	{
		return $this->context;
	}

	/**
	 *
	 * @return BL_Event_Dispatcher
	 */
	public function getEventDispatcher()
	{
		return $this->eventDispatcher;
	}

	/**
	 *
	 * @deprecated
	 * @param object|NULL $subject
	 * @param string $name
	 * @param array $parameters
	 * @return BL_Event
	 */
	public function event($subject, $name, array $parameters = array())
	{
		return new BL_Event($subject, $name, $parameters);
	}

	protected function dispatchModule()
	{
		$module = $this->request->request('m', $this->settings['defaultModule']);
		$this->module = explode('.', $module);
		$module = $this->module[0];

		$this->getEventDispatcher()->notify($this->event($this, 'system.application.beforeDispatch'));

		if (in_array($module, $this->modules)) {
			$this->executeScheme($module);
		} else {
			$this->getEventDispatcher()->notifyUntil($this->event($this, 'system.application.notDefinedModule', array('module' => $module)));
		}
	}

	protected function executeScheme($module)
	{
		$type = $this->isBackend() ? 'backend' : 'frontend';
		$dir = $this->context->loader()->getDir('modules_' . $type);
		if (!file_exists($schemeFile = "$dir/$module/scheme.php")) {
			$this->getEventDispatcher()->notifyUntil($this->event($this, 'system.application.missingScheme', array('module' => $module)));
		}

		$scheme = include $schemeFile;
		if (!is_array($scheme) || empty($scheme['avalible']) || empty($scheme['default'])) {
			throw new BL_Exception(sprintf('Invalid module scheme %s', $schemeFile));
		}

		$subModule = $scheme['default'];
		if ($_subModule = $this->getActiveModule(1, false)) {
			$subModule = in_array($_subModule, $scheme['avalible']) ? $_subModule : $scheme['default'];
		}
		$fullModule = $this->getActiveModule() . '.' . $subModule;
		//TODO:    
//		if ($_app->getActiveModule(0, false) != $fullModule && $admin) {
//			$_app->getResponse()->redirect("admin/$fullModule");
//		}
		//$_app->setActiveModule($fullModule);
		$this->context->loader()->runModule($fullModule, $this->isBackend());
	}

	/**
	 *
	 * @return BL_Request
	 */
	public function getRequest()
	{
		return $this->request;
	}
	/**
	 *
	 * @return BL_Response
	 */
	public function getResponse()
	{
		return $this->response;
	}
	/**
	 *
	 * @return BL_Page
	 */
	public function getPage()
	{
		return $this->page;
	}

	public function getModules()
	{
		return $this->modules;
	}
	
	public function getAction()
	{
		return $this->request->request('do', $this->settings['defaultAction']);
	}

	public function isAction($action)
	{
		return $this->getAction() == $action;
	}

	public function getActiveForm()
	{
		return $this->request->request($this->settings['activeFormNameField']);
	}

	public function isActiveForm($formName)
	{
		return $formName == $this->getActiveForm();
	}

	public function getActiveModule($index = 0, $onlyPart = true)
	{
		if (empty($this->module[$index]))
			return false;

		return $onlyPart ? $this->module[$index] : implode('.', array_slice($this->module, $index));
	}
	/*
	// for BL_Loader only
	public function setActiveModule($module)
	{
		$this->module = explode('.', $module);
	}
	*/
	public function autoRendering($flag = NULL)
	{
		if (NULL === $flag)
			return $this->autoRender;

		return $this->autoRender = (bool)$flag;
	}

	public function shutDown()
	{
		$this->response->send($this->page->getPage());
	}
}
