<?php
/**
 * Bicycle Libraries
 *
 * Set of libraries wich can help you
 *
 * @package   Bicycle Libraries
 * @author    Konstantin.Myakshin
 * @copyright Copyright (c) 2009-2010, Brouzie, Inc.
 * @license   LGPL
 * @link      http://brouzie.com/projects/bicycle-libraries/
 * @link      http://code.google.com/p/bicycle-libraries/
 * @filesource
 */

// based on sfEventDispatcher by Fabien Potencier
class BL_Event_Dispatcher
{
	protected $dir = '';
	protected $loaded = array();
	protected $listeners = array();

	public function __construct($dir)
	{
		$this->dir = $dir;
	}

	protected function checkListener($name)
	{
		if (!isset($this->listeners[$name])) {
			$this->listeners[$name] = array();
		}

		if (!isset($this->loaded[$name])) {
			$file = $this->dir . '/' . str_replace('.', '/', $name) . '.php';
			if (file_exists($file)) {
				$listeners = include $file;
				is_array($listeners) || $listeners = array();
				foreach ($listeners as $listener) {
					$this->listeners[$name][] = $listener;
				}
			}
			$this->loaded[$name] = true;
		}
	}

	/**
	 * Connects a listener to a given event name.
	 *
	 * @param string  $name      An event name
	 * @param mixed   $listener  A PHP callable
	 */
	public function connect($name, $listener)
	{
		$this->checkListener($name);
		$this->listeners[$name][] = $listener;
	}

	/**
	 * Disconnects a listener for a given event name.
	 *
	 * @param string   $name      An event name
	 * @param mixed    $listener  A PHP callable
	 *
	 * @return bool false if listener does not exist, true otherwise
	 */
	public function disconnect($name, $listener)
	{
		$this->checkListener($name);

		foreach ($this->listeners[$name] as $i => $callable) {
			if ($listener === $callable) {
				unset($this->listeners[$name][$i]);
				return true;
			}
		}
		return false;
	}

	public function subscribe($object)
	{
		//TODO: implement
	}

	public function unsubscribe($object)
	{
		//TODO: implement
	}

		/**
	 * Notifies all listeners of a given event.
	 *
	 * @param  BL_Event $event A sfEvent instance
	 *
	 * @return BL_Event The sfEvent instance
	 */
	public function notify(BL_Event $event)
	{
		foreach ($this->getListeners($event->getName()) as $listener) {
			call_user_func($listener, $event); //TODO: method_exists($listener->on_$eventName)
		}

		return $event;
	}

	/**
	 * Notifies all listeners of a given event until one returns a non null value.
	 *
	 * @param  BL_Event $event A sfEvent instance
	 *
	 * @return BL_Event The sfEvent instance
	 */
	public function notifyUntil(BL_Event $event)
	{
		foreach ($this->getListeners($event->getName()) as $listener) {
			if (call_user_func($listener, $event)) {
				$event->setProcessed(true);
				break;
			}
		}

		return $event;
	}

	/**
	 * Filters a value by calling all listeners of a given event.
	 *
	 * @param  BL_Event $event   A sfEvent instance
	 * @param  mixed    $value   The value to be filtered
	 *
	 * @return BL_Event The BL_Event instance
	 */
	public function filter(BL_Event $event, $value)
	{
		foreach ($this->getListeners($event->getName()) as $listener) {
			$value = call_user_func_array($listener, array($event, $value));
		}

		$event->setReturnValue($value);

		return $event;
	}

	/**
	 * Returns true if the given event name has some listeners.
	 *
	 * @param  string   $name    The event name
	 *
	 * @return bool true if some listeners are connected, false otherwise
	 */
	public function hasListeners($name)
	{
		$this->checkListener($name);
		return (bool)count($this->listeners[$name]);
	}

	/**
	 * Returns all listeners associated with a given event name.
	 *
	 * @param  string   $name    The event name
	 *
	 * @return array  An array of listeners
	 */
	public function getListeners($name)
	{
		$this->checkListener($name);
		$this->listeners[$name] = array_unique($this->listeners[$name]);
		return $this->listeners[$name];
	}

	/**
	 * Constructs a new BL_Event.
	 *
	 * @param object|NULL $subject      The subject
	 * @param string      $name         The event name
	 * @param array       $parameters   An array of parameters
	 * @return BL_Event 
	 */
	public function event($subject, $name, array $parameters = array())
	{
		return new BL_Event($subject, $name, $parameters);
	}
}
