<?php
/**
 * Bicycle Libraries
 *
 * Set of libraries wich can help you
 *
 * @package   Bicycle Libraries
 * @author    Konstantin.Myakshin
 * @copyright Copyright (c) 2009-2010, Brouzie, Inc.
 * @license   LGPL
 * @link      http://brouzie.com/projects/bicycle-libraries/
 * @link      http://code.google.com/p/bicycle-libraries/
 * @filesource
 */

/**
 * Request (GET, POST, COOKIE, FILES, partly SERVER accessor class)
 *
 * @package Bicycle Libraries
 * @author  Konstantin.Myakshin
 * @link    http://brouzie.com/projects/bicycle-libraries/user-guide/request.html
 */
class BL_Request implements ArrayAccess
{
	const GET     = 'GET';
	const POST    = 'POST';

	const REQUEST = 'REQUEST';
	const FILES   = 'FILES';
	const COOKIES = 'COOKIES';
	
	const DEFAULT_VALUE = NULL;
	
	private static $instance = NULL;

	protected $data = array();
	
	protected $isAjax    = false;
	protected $isIframed = false;
	
	protected $storages = array();
	protected $languages = array();

	protected function __construct()
	{
		if (get_magic_quotes_gpc()) {
			arrayFilter($_GET);
			arrayFilter($_POST);
			arrayFilter($_COOKIE);
		}
		
		if (isset($_GET['ajax']) || isset($_POST['ajax'])) {
			$this->isAjax = true;
			$this->isIframed = isset($_GET['iframe']) || isset($_POST['iframe']);
			
			if (BL_APP_CHARSET != 'UTF-8' && !$this->isIframed) {
				$_GET  = iconvArray($_GET);
				$_POST = iconvArray($_GET);
			}
		}
		
		$this->data[self::GET]     = (array)$_GET;
		$this->data[self::POST]    = (array)$_POST;
		$this->data[self::REQUEST] = array_merge($this->data[self::GET], $this->data[self::POST]);
		$this->data[self::COOKIES] = (array)$_COOKIE;
		
		$this->data[self::FILES]   = array();

		foreach ($_FILES as $field => $info) {

			if (is_array($info['error'])) {
				$this->data[self::FILES][$field] = array();
				
				foreach ($info['error'] as $key => $error) {
					$this->data[self::FILES][$field][] = array(
						'name'     => $info['name'][$key],
						'type'     => $info['type'][$key],
						'size'     => $info['size'][$key],
						'tmp_name' => $info['tmp_name'][$key],
						'error'    => $error
					);
				}
			} else {
				$this->data[self::FILES][$field] = $info;
			}
		}
	}
	
	protected function __clone() {}

	/**
	 *
	 * @param string $type
	 * @return BL_Request_Storage|BL_Request_StorageCookie|BL_Request_StorageFiles
	 */
	protected function getStorage($type)
	{
		$storage = 'BL_Request_Storage';
		$type == self::COOKIES && $storage = 'BL_Request_StorageCookie';
		$type == self::FILES   && $storage = 'BL_Request_StorageFiles';
		return isset($this->storages[$type]) ? $this->storages[$type] : new $storage($this, $type);
	}

	/**
	 *
	 * @return BL_Request
	 */
	public static function instance()
	{
		return self::$instance ? self::$instance : self::$instance = new self;
	}
	
	protected function getVariable($type, $key, $default = self::DEFAULT_VALUE)
	{
		if ($key) {
			return empty($this->data[$type][$key]) ? $default : $this->data[$type][$key];
		}
		return $this->data[$type];
	}
	
	public function deploy(array $data, $type = self::GET)
	{
		foreach ($data as $key => $value) {
			$this->set($key, $value, $type);
		}
		
		return $this;
	}
	//TODO: set|get state
	//TODO: what to do if key already exists
	//TODO???: lock
	public function set($key, $value, $type = self::POST)
	{
		if ($type == self::GET || $type == self::POST) {
			$this->data[$type][$key] = $value;
			$this->data[self::REQUEST][$key] = $value;
		} else {
			throw new BL_Exception('Unknown type');
		}
		
		return $this;
	}
	
	public function get($key = NULL, $default = self::DEFAULT_VALUE)
	{
		return $this->getVariable(self::GET, $key, $default);
	}
	
	public function post($key = NULL, $default = self::DEFAULT_VALUE)
	{
		return $this->getVariable(self::POST, $key, $default);
	}
	
	public function request($key = NULL, $default = self::DEFAULT_VALUE)
	{
		return $this->getVariable(self::REQUEST, $key, $default);
	}

	public function files($key = NULL)
	{
		return $this->getVariable(self::FILES, $key, array());
	}

	public function cookies($key = NULL, $default = self::DEFAULT_VALUE)
	{
		$this->data[self::COOKIES] = array_merge($this->data[self::COOKIES], BL_Response::instance()->getCookies());
		return $this->getVariable(self::COOKIES, $key, $default);
	}
	
	public function __get($key)
	{
		if (in_array(strtoupper($key), array(self::GET, self::POST, self::REQUEST, self::COOKIES, self::FILES))) {
			return $this->getStorage(strtoupper($key));
		}
		
		return $this->getVariable(self::REQUEST, $key, self::DEFAULT_VALUE);
	}
	
	public function __set($key, $value)
	{
		if (in_array(strtoupper($key), array(self::GET, self::POST, self::REQUEST, self::COOKIES, self::FILES))) {
			throw new BL_Exception('Reserved');
		}
		$this->set($key, $value);
	}

	public function offsetExists($offset)
	{
		return $this->has($key);
	}

	public function offsetGet($offset)
	{
		if (in_array(strtoupper($offset), array(self::GET, self::POST, self::REQUEST, self::COOKIES, self::FILES))) {
			return $this->getStorage(strtoupper($offset));
		}

		return $this->getVariable(self::REQUEST, $offset, self::DEFAULT_VALUE);
	}
	
	public function offsetSet($offset, $value)
	{
		if (in_array(strtoupper($offset), array(self::GET, self::POST, self::REQUEST, self::COOKIES, self::FILES))) {
			throw new BL_Exception('Reserved');
		}
		$this->set($offset, $value);
	}

	public function offsetUnset($offset)
	{
		throw new BL_Exception('Reserved');
	}
	
	public function has($key, $type = self::REQUEST)
	{
		return isset($this->data[$type][$key]);
	}

	/**
	 * @author Symfony team
	 *
	 * @return string
	 */
	public function isSecure()
	{
		return (
			(isset($_SERVER['HTTPS']) && (strtolower($_SERVER['HTTPS']) == 'on' || $_SERVER['HTTPS'] == 1))
			||
			(isset($_SERVER['HTTP_SSL_HTTPS']) && (strtolower($_SERVER['HTTP_SSL_HTTPS']) == 'on' || $_SERVER['HTTP_SSL_HTTPS'] == 1))
			||
			(isset($_SERVER['HTTP_X_FORWARDED_PROTO']) && strtolower($_SERVER['HTTP_X_FORWARDED_PROTO']) == 'https')
		);
	}
	
	public function getRequestMethod()
	{
		return isset($_SERVER['REQUEST_METHOD']) ? $_SERVER['REQUEST_METHOD'] : 'CLI';
	}
	
	public function getIP()
	{
		return $_SERVER['REMOTE_ADDR'];
	}
	
	public function isMethod($method)
	{
		return strtoupper($method) == $this->getMethod();
	}
	
	public function getReferer()
	{
		return isset($_SERVER['HTTP_REFERER']) ? $_SERVER['HTTP_REFERER'] : '';
	}

	public function getRequestUri()
	{
		return $_SERVER['REQUEST_URI'];
	}

	/**
	 * Splits an HTTP header for the current web request.
	 * @author Symfony team
	 *
	 * @param string $header  Header to split
	 */
	public function splitHttpAcceptHeader($header)
	{
		$values = array();
		foreach (array_filter(explode(',', $header)) as $value) {
			// cut off any q-value that might come after a semi-colon
			if ($pos = strpos($value, ';')) {
				$q     = (float) trim(substr($value, $pos + 3));
				$value = trim(substr($value, 0, $pos));
			} else {
				$q = 1;
			}

			$values[$value] = $q;
		}

		arsort($values);

		return array_keys($values);
  }

	/**
	 * Gets a list of languages acceptable by the client browser
	 * @author Symfony team
	 *
	 * @return array Languages ordered in the user browser preferences
	 */
	public function getLanguages()
	{
		if ($this->languages) {
			return $this->languages;
		}

		if (!isset($_SERVER['HTTP_ACCEPT_LANGUAGE'])) {
			return array();
		}

		$languages = $this->splitHttpAcceptHeader($_SERVER['HTTP_ACCEPT_LANGUAGE']);
		foreach ($languages as $lang) {
			if (strstr($lang, '-')) {
				$codes = explode('-', $lang);
				if ($codes[0] == 'i') {
					// language not listed in ISO 639 that are not variants
					// of any listed language, which can be registerd with the
					// i-prefix, such as i-cherokee
					if (count($codes) > 1) {
						$lang = $codes[1];
					}
				} else {
					for ($i = 0, $max = count($codes); $i < $max; $i++) {
						if ($i == 0) {
							$lang = strtolower($codes[0]);
						} else {
							$lang .= '_'.strtoupper($codes[$i]);
						}
					}
				}
			}

			$this->languages[] = $lang;
		}

		return $this->languages;
	}

	public function isPost()
	{
		return $this->getRequestMethod() == self::POST;
	}

	public function isAjax()
	{
		return $this->isAjax;
	}
	
	public function isIframed()
	{
		return $this->isIframed;
	}
}
