<?php
/**
 * Bicycle Libraries
 *
 * Set of libraries wich can help you
 *
 * @package   Bicycle Libraries
 * @author    Konstantin.Myakshin
 * @copyright Copyright (c) 2009-2010, Brouzie, Inc.
 * @license   LGPL
 * @link      http://brouzie.com/projects/bicycle-libraries/
 * @link      http://code.google.com/p/bicycle-libraries/
 * @filesource
 */

class BL_Response
{
	const TYPE_TEXT_HTML = 'text/html';
	const TYPE_TEXT_CSS  = 'text/css';
	
	const TYPE_APP_JS    = 'application/javascript';
	const TYPE_APP_JSON  = 'application/json';

	const TYPE_APP_RSS   = 'application/rss+xml';
	const TYPE_APP_ATOM  = 'application/atom+xml';
	const TYPE_APP_RDF   = 'application/rdf+xml';

	/**
	 *
	 * @var BL_Response
	 */
	private static $instance;
	/**
	 *
	 * @var BL_Request
	 */
	protected $request;

	protected $codes = array(
		array(
			200 => 'OK',
			201 => 'Created',
			202 => 'Accepted',
			204 => 'No Content',
			
			300 => 'Multiple Choices',
			301 => 'Moved Permanently',
			302 => 'Found',
			304 => 'Not Modified',
			306 => '(Unused)',
			307 => 'Temporary Redirect',

			400 => 'Bad Request',
			401 => 'Unauthorized',
			403 => 'Forbidden',
			404 => 'Not Found',
			416 => 'Requested Range Not Satisfiable',
			417 => 'Expectation Failed',
			
			500 => 'Internal Server Error',
			501 => 'Not Implemented',
			502 => 'Bad Gateway',
			503 => 'Service Unavailable',
		),
		array(
			100 => 'Continue',
			101 => 'Switching Protocols',

			203 => 'Non-Authoritative Information',
			205 => 'Reset Content',
			206 => 'Partial Content',

			303 => 'See Other',
			305 => 'Use Proxy',

			402 => 'Payment Required',
			405 => 'Method Not Allowed',
			406 => 'Not Acceptable',
			407 => 'Proxy Authentication Required',
			408 => 'Request Timeout',
			409 => 'Conflict',
			410 => 'Gone',
			411 => 'Length Required',
			412 => 'Precondition Failed',
			413 => 'Request Entity Too Large',
			414 => 'Request-URI Too Long',
			415 => 'Unsupported Media Type',

			504 => 'Gateway Timeout',
			505 => 'HTTP Version Not Supported',
		)
	);

	//TODO: serialize/unserialize (set state)
	protected $headers = array();

	protected $cookies = array();

	protected $code = 200;
	protected $contentType = self::TYPE_TEXT_HTML;
	protected $charset = BL_APP_CHARSET;
	protected $body = '';

	protected $settings = array();

	protected function __construct(BL_Request $request)
	{
		//TODO: ob_start
		$this->request = $request;

		$defaultSettings = array(
			'cookies' => array(
				'expire'   => 86400,
				'path'     => $this->getPath(),
				'domain'   => $this->getDomain(),
				'secure'   => $this->request->isSecure(),
				'httpOnly' => true
			),
			'redirectFailHandler' => false //TODO: use events
		);
		$this->settings = BL_Settings::getMerged($defaultSettings, 'application', 'response');

		$this->headers['Content-type'] = $this->contentType;
	}
	
	protected function  __clone() {}
	//TODO: move out (to Request?)
	protected function getDomain($url = SITE_URL)
	{
		$url = explode('/', $url);
		return $url[2];
	}
	//TODO: move out (to Request?)
	protected function getPath($url = SITE_URL)
	{
		$url = explode('/', $url);
		return '/' . implode('/', array_slice($url, 3));
	}

	/**
	 *
	 * @return BL_Response
	 */
	public static function instance(BL_Request $request = NULL)
	{
		return self::$instance ? self::$instance : self::$instance = new self($request);
	}
	
	public function body($body = NULL)
	{
		if (NULL === $body) {
			return $this->body;
		}

		$this->body = $body;
		
		return $this;
	}
	
	public function code($code = NULL)
	{
		if (NULL === $code) {
			return $this->code;
		}

		if (isset($this->codes[0][$code]) || isset($this->codes[1][$code]) ) {
			$this->code = $code;
		} else {
			throw new BL_Exception('Unknown HTTP status code');
		}
		return $this;
	}
	
	public function contentType($contentType = NULL)
	{
		if (NULL === $contentType) {
			return $this->contentType;
		}

		$this->contentType = $contentType;
		$this->headers['Content-type'] = $contentType;
		
		return $this;
	}

	public function charset($charset = NULL)
	{
		if (NULL === $charset) {
			return $this->charset;
		}

		$this->charset = $charset;

		return $this;
	}

	protected function checkHeadersSent($failHandler = false)
	{
		$file = '';
		$line = 0;
		if (headers_sent($file, $line)) {
			if (is_callable($failHandler)) {
				call_user_func_array($failHandler, array($url, $file, $line));
			} else {
				throw new BL_Exception(sprintf('Headers already sent on file %s and line %d', $file, $line));
			}
		}
	}

	public function sendHeaders($useCheck = true)
	{
		$useCheck && $this->checkHeadersSent();

		foreach ($this->headers as $type => $header) {
			if ($type == 'Content-type') {
				header('Content-type: ' . $header . '; charset=' . $this->charset);
			} else {
				header($type . ': ' . $header);
			}
		}
		
		if (isset($this->codes[0][$this->code])) {
			header('HTTP/1.0 ' . $this->code . ' ' . $this->codes[0][$this->code]);
		} else {
			header('HTTP/1.1 ' . $this->code . ' ' . $this->codes[1][$this->code]);
		}

		return $this;
	}

	public function sendBody($body = NULL)
	{
		echo NULL === $body ? $this->body : $body;
	}

	public function send($body = NULL)
	{
		$this->clearBuffer();
		$this->sendHeaders();
		$this->sendBody($body);
		exit;
	}
	
	public function setCookie($name, $value, $expire = NULL, $path = NULL, $domain = NULL, $secure = NULL, $httpOnly = NULL)
	{
		$this->checkHeadersSent();
		
		if (NULL === $expire) {
			// use pre-defined value from settings
			$expire = $this->settings['cookies']['expirie'];
		} elseif (0 === (int)$expire) {
			// session
			$expire = 0;
		} elseif (is_numeric($expire)) {
			$expire = (int)$expire;
			// real time or delta time
			abs($expire) > 10 * 365 * 86400 || $expire = time() + $expire;
		} else {
			$expire = strtotime($expire);
			if (false === $expire || -1 === $expire) {
				throw new BL_Exception('Your expire parameter is not valid.');
			}
			// real time or delta time
			abs($expire) > 10 * 356 * 84400 || $expire = time() + $expire;
		}
		
		$path = NULL === $path ? $this->settings['cookies']['path'] : $this->getPath() . $path;
		// apply '/' to the end of path
		'/' == substr($path, -1) || $path .= '/';

		NULL === $domain   && $domain   = $this->settings['cookies']['domain'];
		NULL === $secure   && $secure   = $this->settings['cookies']['secure'];
		NULL === $httpOnly && $httpOnly = $this->settings['cookies']['httpOnly'];

		$check = strpos($this->getDomain(), $domain);
		//TODO: check this part again
		//TODO: check path
		if (is_int($check) && $check >= 0 && ($check + strlen($this->getDomain())) <= strlen($domain)) {
			$this->cookies[$name] = $value;
			setrawcookie($name, $value, $expire, $path, $domain, $secure, $httpOnly);
		}
		return $this;
	}
	
	public function getCookies()
	{
		return $this->cookies;
	}
	
	public function removeCookie($name, $path = NULL, $domain = NULL)
	{
		if (is_array($name)) {
			foreach ($name as $_name) {
				$this->setCookie($_name, '', -99999, $path, $domain);
			}
		} else {
			$this->setCookie($name, '', -99999, $path, $domain);
		}
		
		return $this;
	}
	
	public function ajaxResponse($body, $type = 'json', $cache = false)
	{
		$cache || $this->noCache();
		
		switch ($type) {
			case 'json':
				if ($this->request->isIframed()) {
					$this
						->contentType(self::TYPE_TEXT_HTML)
						->send(htmlspecialchars(json($body)));
				}
				$this
					->contentType(self::TYPE_APP_JSON)
					->send(json($body));
			//TODO: XML
			default:
				$this
					->contentType(self::TYPE_TEXT_HTML)
					->send($body);
		}
	}

	function noCache()
	{
		$this->checkHeadersSent();
		header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');
		header('Last-Modified: ' . gmdate('D, d M Y H:i:s') . ' GMT');
		header('Cache-Control: no-store, no-cache, must-revalidate');
		header('Cache-Control: post-check=0, pre-check=0', false);
		header('Pragma: no-cache');
	}

	protected function clearBuffer()
	{
		while (ob_get_level())
			ob_end_clean();
	}

	public function redirect($url, $code = 302)
	{
		if (empty($url))
			throw new BL_Exception('Empty url for redirect');
		
		// backward compatibility && URL not absolute
		if ($url[0] != '/' && strpos($url, '://') === false) {
			$url = SITE_URL . $url;
		}

		$this->clearBuffer();
		$this->headers = array(
			'Location' => $url
		);
		$this->noCache();
		$this->code($code);
		//TODO: replace with event
		$this->checkHeadersSent($this->settings['redirectFailHandler']);
		$this->sendHeaders(false);
		exit;
	}
	//TODO: setRedirect
}

