<?php

class BL_Session extends BL_Session_Light
{
	protected $options = array(
		'name'    => 'BLSID',
		'gc'      => array(
			'probability' => 0.2,
			'lifetime'    => 1440
		),
		'randomizer' => array('self', 'randomize'),
		'cookies' => array(
			'expire'   => 0,
			'path'     => NULL,
			'domain'   => NULL,
			'secure'   => NULL,
			'httpOnly' => true
		),
		#'checks' => array('referer', 'ip', 'ua'),
		'storage'    => array(
			'class'   => 'file',
			'options' => array()
		),
		'serializer' => 'default',
	);

	/**
	 *
	 * @var BL_Request
	 */
	protected $request;

	/**
	 *
	 * @var BL_Response
	 */
	protected $response;

	/**
	 *
	 * @var BL_Session_StorageAdapterInterface
	 */
	protected $storage;

	/**
	 *
	 * @var BL_Session_SerializeAdapterInterface
	 */
	protected $serializer;

	/**
	 *
	 * @var BL_Event_Dispatcher
	 */
	protected $eventDispatcher;

	protected function __clone()
	{}

	public function __construct(BL_Request $request, BL_Response $response, array $options = array(), BL_Event_Dispatcher $eventDispatcher = NULL)
	{
		$this->request  = $request;
		$this->response = $response;
		$this->options  = array_merge_recursive_distinct($this->options, $options);
		$this->eventDispatcher = $eventDispatcher;
		
		$this->prepareStorage();
		$this->prepareSerializer();
		$this->generateSession();
		$this->loadData();

		if (!is_numeric($prob = $this->options['gc']['probability']) || $prob >= 1 || $prob <= 0)
			throw new BL_Exception ('Bad gc.probability');
		//TODO: move out
		$max = pow(10, strlen($prob) - 2); // 10 ^ count of num after point
		(mt_rand(1, $max) <= $prob * $max) && $this->gc();
	}

	protected function prepareStorage()
	{
		if (is_string($storage = $this->options['storage'])) {
			$storage = new $storage();
		} elseif (is_array($storage)) {
			$options = $storage['options'];
			$storage = $storage['class'];
			class_exists($storage) || class_exists($storage = 'BL_Session_StorageAdapter_' . ucfirst($storage));
			$storage = new $storage($options);
		}

		if (!$storage instanceof BL_Session_StorageAdapterInterface) {
			throw new BL_Exception('Bad storage');
		}
		$this->storage = $storage;
	}

	protected function prepareSerializer()
	{
		if (is_string($serializer = $this->options['serializer'])) {
			class_exists($serializer) || class_exists($serializer = 'BL_Session_SerializeAdapter_' . ucfirst($serializer));
			$serializer = new $serializer();
		}
		if (!$serializer instanceof BL_Session_SerializeAdapterInterface) {
			throw new BL_Exception('Bad serializer');
		}
		$this->serializer = $serializer;
	}

	public static function randomize()
	{
		return sha1(mt_rand(-9999, 9999) . microtime(true));
	}

	public function regenerateId()
	{
		$this->storage->destroy($id = $this->id);
		$this->id = call_user_func($this->options['randomizer']);
		$this->generateSession();
		return $id;
	}

	protected function generateSession()
	{
		$key = $this->options['name'];

		$this->id || $this->id = $this->request->cookies($key);
		$this->id || $this->id = $this->request->request($key);
		$this->id || $this->id = call_user_func($this->options['randomizer']);
		
		$o = $this->options['cookies'];
		$this->response->setCookie($key, $this->id, $o['expire'], $o['path'], $o['domain'], $o['secure'], $o['httpOnly']);
	}

	protected function loadData()
	{
		$this->data = $this->serializer->unserialize($this->storage->read($this->id, $this->options['gc']['lifetime']));
	}

	/**
	 *
	 * @return BL_Session_SerializeAdapterInterface
	 */
	public function getSerializer()
	{
		return $this->serializer;
	}

	/**
	 *
	 * @param string|NULL $id
	 * @return BL_Session|string
	 */
	public function id($id = NULL)
	{
		if ($id) {
			$this->storage->destroy($this->id);
			$this->id = $id;
			$this->generateSession();
			return $this;
		}
		return $this->id;
	}

	public function commit()
	{
		$this->storage->write($this->id, $this->serializer->serialize($this->data));
		return $this;
	}

	public function destroy()
	{
		if ($this->eventDispatcher) {
			$this->eventDispatcher->notify($this->eventDispatcher->event($this, 'system.session.destroy'));
		}

		$this->data = array();
		$this->gc($this->id);
		$this->regenerateId();
		return $this;
	}

	public function destroyNs($name)
	{
		$this->ns($name)->flush();
		unset($this->namespaces[$name]);
		return $this;
	}

	protected function gc($id = NULL)
	{
		foreach ($this->storage->getExpiried($this->options['gc']['lifetime']) as $id) {
			if ($this->eventDispatcher) {
				$session = new BL_Session_Light($this, $id, $this->serializer->unserialize($this->storage->read($id, $ttl)));
				$this->eventDispatcher->notify($this->eventDispatcher->event($session, 'system.session.gc'));
			}
			$this->storage->destroy($id);
		}
	}

	public function __destruct()
	{
		$this->commit();
	}
}
