<?php
/**
 * Bicycle Libraries
 *
 * Set of libraries wich can help you
 *
 * @package   Bicycle Libraries
 * @author    Konstantin.Myakshin
 * @copyright Copyright (c) 2009-2010, Brouzie, Inc.
 * @license   LGPL
 * @link      http://brouzie.com/projects/bicycle-libraries/
 * @link      http://code.google.com/p/bicycle-libraries/
 * @filesource
 */

class BL_Validator
{
	// field => array(rule1 => array(messages, options))
	protected $fields = array();
	
	// needed for getErrors and getError
	protected $errors = array();
	// needed for getValues and getValue
	protected $values = array();
	

	protected static $firstRun = true;

	protected static $validators = array();

	protected static $instances = array();

	public function __construct($formName)
	{
		if (isset(self::$instances[$formName])) {
			throw new BL_Exception(sprintf('Validator for form %s already exists', $formName));
		}
		self::$instances[$formName] = $this;
		
		if (self::$firstRun) {
			$this->attachValidator(new BL_Validator_Core());
			self::$firstRun = false;
			
			if ($settings = BL_Settings::get('validator')) {
				// NB! you cann't autoload validator wich get args using his constructor
				foreach ($settings['autoload'] as $validator) {
					$this->attachValidator(new $validator());
				}
			}
		}
	}
	
	protected function __clone() {}

	/**
	 *
	 * @param string $formName
	 * @return BL_Validator
	 */
	public static function instance($formName)
	{
		return isset(self::$instances[$formName]) ? self::$instances[$formName] : self::$instances[$formName] = new self($formName);
	}

	public static function destruct($formName)
	{
		if (isset(self::$instances[$formName]))
			unset(self::$instances[$formName]);
	}
	/**
	 *
	 * @param <type> $field
	 * @param <type> $rule
	 * @param <type> $messages
	 * @param <type> $options
	 * @return BL_Validator
	 */
	public function addRule($field, $rule = 'use', $messages = array(), $options = array())
	{
		//TODO: is_array($rule)
		if (!isset($this->fields[$field]))
			$this->fields[$field] = array();
		
		$this->fields[$field][$rule] = array((array)$messages, (array)$options);
		return $this;
	}
	
	public function validate($data, array &$values = array(), array &$errors = array())
	{
		// reset
		$this->errors = $errors = array();
		$this->values = $values = array();

		foreach ($this->fields as $field => $rules) {
			$values[$field] = (isset($data[$field])) ? $data[$field] : '';
			$trim = !isset($rules['noTrim']);
			$values[$field] = ($trim && !is_array($values[$field])) ? trim($values[$field]) : $values[$field];

			$required = isset($rules['notEmpty']);
			// check empty at first
			$validator = $this->getValidator('notEmpty');
			$additional = $required ? $rules['notEmpty'] : array(array(), array());
			$error = '';
			if (!$validator->validate('notEmpty', $field, $values[$field], $error, $additional[0], $additional[1])) {
				if ($required) {
					// store error msg
					$this->errors[$field] = $errors[$field] = $error;
				}
				$this->values[$field] = $values[$field];
				// field is empty. No any others checks needed
				if (!isset($rules['noSkipOnEmpty']))
					continue;
			}
			
			foreach ($rules as $rule => $additional) {
				// skip this rules because we have already checked they above
				if (in_array($rule, array('noTrim', 'notEmpty', 'use', 'noSkipOnEmpty')))
					continue;
				
				$validator = $this->getValidator($rule);
				$error = '';
				if (!$validator->validate($rule, $field, $values[$field], $error, $additional[0], $additional[1])) {
					// store error msg
					$this->errors[$field] = $errors[$field] = $error;
					// and skip any other checks for this field
					break;
				}
			} // endforeach $rules
			
			$this->values[$field] = $values[$field];
		} // endforeach $this->rules

		return !(bool)$this->errors;
	}

	public function getErrors()
	{
		return $this->errors;
	}

	public function getError($field)
	{
		return isset($this->errors[$field]) ? $this->errors[$field] : false;
	}

	public function getValues()
	{
		return $this->values;
	}

	public function getValue($field, $default = '')
	{
		return isset($this->values[$field]) ? $this->values[$field] : $default;
	}

	public static function attachValidator(BL_Validator_Abstract $validator)
	{
		foreach ($validator->getRules() as $rule) {
			self::$validators[$rule] = $validator;
		}
	}

	public static function getValidatorInstance($className)
	{
		foreach (self::$validators as $validator) {
			if (get_class($validator) == $className)
				return $validator;
		}
		throw new BL_Exception('Unknown class validator');
	}
	
	/**
	 *
	 * @param string $rule
	 * @return BL_Validator_Abstract
	 */
	protected function getValidator($rule)
	{
		if (isset(self::$validators[$rule])) {
			self::$validators[$rule]->setValidator($this);
			return self::$validators[$rule];
		}

		throw new BL_Exception(sprintf('Validator for rule %s doesn\'t exists', $rule));
	}
}
