<?php
/**
 * Bicycle Libraries
 *
 * Set of libraries wich can help you
 *
 * @package   Bicycle Libraries
 * @author    Konstantin.Myakshin
 * @copyright Copyright (c) 2009-2010, Brouzie, Inc.
 * @license   LGPL
 * @link      http://brouzie.com/projects/bicycle-libraries/
 * @link      http://code.google.com/p/bicycle-libraries/
 * @filesource
 */

class BL_Loader
{
	/**
	 *
	 * @var BL_Loader
	 */
	private static $instance = NULL;

	protected $dirs = array(
		'core'             => BL_CORE_DIR,
		'vendors'          => 'code/vendors/',
		'local'            => 'code/local/',
		'modules_backend'  => 'code/modules/backend/',
		'modules_frontend' => 'code/modules/frontend/',
	);

	protected $vendors = array();

	/**
	 *
	 * @var BL_Application
	 */
	protected $app = NULL;

	/**
	 *
	 * @var BL_Context
	 */
	protected $context = NULL;

	protected function __construct(array $dirs = array())
	{
		$this->dirs = array_merge($this->dirs, $dirs);
	}

	/**
	 *
	 * @param string $name
	 * @return BL_Loader
	 */
	public static function instance()
	{
		if (self::$instance)
			return self::$instance;

		throw new BL_Loader_Exception('Loader not ready. Use `BL_Loader::autoloadRegister()` first');
	}

	public function getDirs()
	{
		return $this->dirs;
	}

	public function getDir($type)
	{
		return $this->dirs[$type];
	}

	/**
	 *
	 * @param array $dirs
	 * @return BL_Loader
	 */
	public static function autoloadRegister(array $dirs = array())
	{
		if (self::$instance)
			throw new BL_Exception ('Already registred');

		ini_set('unserialize_callback_func', 'spl_autoload_call');
		spl_autoload_register(array($obj = new self($dirs), 'autoload'));
		set_include_path(realpath(getcwd()) . PATH_SEPARATOR . realpath($obj->dirs['vendors'])); // for ZF and others non-autoload libs
		return self::$instance = $obj;
	}

	/**
	 *
	 * @param string $prefix
	 * @return BL_Loader
	 */
	public function registerVendor($prefix)
	{
		$this->vendors[] = $prefix;
		return $this;
	}

	/**
	 *
	 * @param BL_Application $app
	 * @return BL_Loader
	 */
	public function injectApp(BL_Application $app)
	{
		$this->app = $app;
		return $this;
	}

	/**
	 *
	 * @param BL_Context $context
	 * @return BL_Loader
	 */
	public function injectContext(BL_Context $context)
	{
		$this->context = $context;
		return $this;
	}

	public function autoload($className)
	{
		try {

			// core libraries
			if ('BL_' == substr($className, 0, 3)) {
				$file = $this->dirs['core'] . str_replace('_', '/', substr($className, 3)) . '.php';
				if (file_exists($file)) {
					include_once $file;
				}

				if (!class_exists($className, false) && !interface_exists($className, false)) {
					throw new BL_Loader_Exception(sprintf('Missing file "%s" for core class "%s"', $file, $className));
				}
				return true;
			}

			//TODO: BLO_ (local)

			// modules
			if ('BLM_' == substr($className, 0, 4)) {
				return $this->loadModule(substr($className, 4));
			}

			// admin modules
			if ('BLAM_' == substr($className, 0, 5)) {
				return $this->loadModule(substr($className, 5), true);
			}

			foreach ($this->vendors as $prefix) {
				if (stripos($className, $prefix) === 0) {
					$file = $this->dirs['vendors'] . str_replace('_', '/', $className) . '.php';
					if (file_exists($file)) {
						include_once $file;
						return true;
					}
					throw new BL_Loader_Exception(sprintf('Missing file "%s" for vendor "%s"', $file, $prefix));
				}
			}

			return false;

		} catch (Exception $e) {
			$this->throwAutoloadException($className, $e);
		}
	}

	/**
	*
	* @author emailmatthijs at gmail dot com
	*
	* @param string $className
	* @param Exception $e
	*/
	protected function throwAutoloadException($className, Exception $e)
	{
		// the use of eval is risky, because it could be easily exploited. To prefend
		// that the use of eval in this method could be exploited we make sure that the
		// variable contains only the allowed characters of a class name.
		if (preg_match('/^[a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*$/', $className)) {
			eval('class ' . $className.'
			{
				static public function throwException()
				{
					throw unserialize(\'' . serialize($e) . '\');
				}
			}
			' . $className . '::throwException();');
		}
	}

	public function moduleClass($moduleName)
	{
		return str_replace(' ', '_', ucwords(str_replace('.', ' ', $moduleName)));
	}

	public function widgetClass($widgetName)
	{
		$widgetName = str_replace('/', '.', $widgetName);
		return $this->moduleClass($widgetName);
	}

	public function moduleFile($moduleName)
	{
		$file = $this->moduleClass($moduleName);
		$file = strtolower($file[0]) . substr($file, 1);
		return str_replace('_', '/', $file);
	}

	public function widgetFile($widgetName)
	{
		list($module, $widget) = explode('/', $widgetName);
		$file = $module . '/widgets/' . $this->widgetClass($widget);
		return str_replace('_', '/', $file);
	}

	public function loadModule($moduleName, $admin = false)
	{
		$type = $admin ? 'backend' : 'frontend';
		$dir = $this->dirs['modules_' . $type];

		$file = $dir . $this->moduleFile($moduleName) . '.php';
		if (file_exists($file)) {
			include_once $file;
			return true; //TODO: check class_exists
		}

		throw new BL_Loader_Exception(sprintf('Missing module file "%s"', $file));
	}

	public function loadWidget($widgetName)
	{
		if (false === strpos($widgetName, '/')) {
			throw new BL_Loader_Exception(sprintf('Bad widget format. Wait for "module/widgetName" but "%s" given', $widgetName));
		}

		$file = $this->dirs['modules_frontend'] . $this->widgetFile($widgetName) . '.php';
		if (file_exists($file)) {
			include_once $file;
			return true;
		}

		throw new BL_Loader_Exception(sprintf('Missing widget file "%s"', $file));
	}

	public function runModule($moduleName, $admin = false)
	{
		if (!$this->app)
			throw new BL_Loader_Exception('Application not ready yet. Use injectApp method');

		$prefix = $admin ? 'BLAM_' : 'BLM_';
		$className = $prefix . $this->moduleClass($moduleName);
		class_exists($className, false) || $this->loadModule($moduleName, $admin);

		$implements = class_implements($className);
		if (in_array('BL_Application_ModuleInterface', $implements)) {
			$moduleObj = new $className($this->app);
		} elseif (in_array('BL_Application_ModuleModernInterface', $implements)) {
			$moduleObj = new $className($this->context);
		} else {
			throw new BL_Loader_Exception($className . ' does not implements interface "BL_Application_ModuleInterface" or "BL_Application_ModuleModernInterface"');
		}

		$moduleObj->process();
	}

	public function execWidget($widgetName, $settings = array())
	{
		if (!$this->app)
			throw new BL_Loader_Exception('Application not ready yet. Use injectApp method');

		$className = 'BLW_' . $this->widgetClass($widgetName);
		class_exists($className, false) || $this->loadWidget($widgetName);

		$implements = class_implements($className);
		if (in_array('BL_Application_WidgetInterface', $implements)) {
			$widgetObj = new $className($this->app, $settings);
		} elseif (in_array('BL_Application_WidgetModernInterface', $implements)) {
			$widgetObj = new $className($this->context, $settings);
		} else {
			throw new BL_Loader_Exception($className . ' does not implements interface "BL_Application_WidgetInterface" or "BL_Application_WidgetModernInterface"');
		}

		return $widgetObj->process();
	}
}
